<TeXmacs|1.0.4.5>

<style|literate-programming>

<\body>
  <doc-data|<doc-title|Literate Programming in
  <TeXmacs>>|<doc-author-data|<author-name|Felix
  Breuer>|<author-email|felix@fbreuer.de>>>

  <section|Overview>

  This is an early prototype for simple (Reverse) Literate Programming
  support in <TeXmacs>. I do not claim that this is the best
  conceptualization of LP and I do not claim that it is implemented in the
  best way. But suits my purposes and it works! As a matter of fact, the
  source code for this LP system can be generated from this very <TeXmacs>
  document!

  So if you want to know how this looks, jump to Implementation section below
  and read on later.

  <paragraph|What LP Functionality is Provided?>I provide four new
  environments, defined in <verbatim|literate-programming.ts>. These are

  <\description>
    <item*|block>A block of code, consisting of a name and one or more lines
    of code. These lines can also contain <strong|inc> or <strong|cmt>
    environments.

    <item*|inc>An inclusion of another block of code, referenced by a name.
    When the output is generated, the referenced block is pasted in place of
    the <strong|inc> environment. Each line of the referenced block is
    indented as much as the <strong|inc> environment.

    <item*|cmt>A comment. This is not reproduced in the generated files.

    <item*|file>A <strong|block> whose name is interpreted as the name of a
    file to be generated.
  </description>

  Examples for all of these can be found below. This conceptualization of LP
  is heavily inspired by <name|Leo>.

  <paragraph|How Does the Code Generation Work?>The code is generated in four
  steps, one of which can be omitted, when some minor bugs in <TeXmacs> are
  fixed. The following steps assume, that you have unpacked the LP archive in
  a subdirectory called <verbatim|LP>.

  Assuming you have a <TeXmacs> document that uses the environments mentioned
  above:

  <\enumerate-numeric>
    <item>Export the document as XML, producing a <verbatim|.tmml> file, say
    <verbatim|tmp1.tmml>.

    <item>Apply the <name|Ruby> script <verbatim|<verbatim|preprocess.rb>> to
    it, generating an XML file:\ 

    <verbatim|ruby LP/preprocess.rb tmp1.tmml tmp2.xml>

    This step can be dropped in the future.

    <item>Apply the <name|XSLT> stylesheet <verbatim|tmml2mycode.xsl>, by
    running

    <verbatim|xsltproc -o tmp3.xml LP/tmml2mycode.xsl tmp2.xml>

    <item>Run the <name|Ruby> script <verbatim|myx2code.rb> via

    <verbatim|rubyx myx2code.rb tmp3.xml>

    this generates all the files that were specified in the <strong|file>
    environments in your document in the current directory.
  </enumerate-numeric>

  When you apply these four steps to this document you should obtain files
  <verbatim|tmml2mycode.xsl>, <verbatim|preprocess.rb> and
  <verbatim|myx2code.rb> that are identical to those contained in the
  archive.

  Alternatively you can use this simple shell script

  <\file|gen.sh>
    #!/bin/sh

    mkdir -p out

    ruby preprocess.rb $1 out/tmp2.xml

    xsltproc -o out/tmp3.xml tmml2mycode.xsl out/tmp2.xml

    cd out

    ruby ../myx2code.rb tmp3.xml
  </file>

  \;

  by running

  <block-anon|./gen.sh path/to/your/doc.tmml>

  from within the archive directory.

  <paragraph|Why so Many Steps?>

  <\enumerate-numeric>
    <item>Sure, all of this could be done from withing <TeXmacs> via
    <name|Guile>. The problem is, that I have never managed to get a grip on
    <TeXmacs> <name|Guile> interface, and I have given up.

    <item>I convert to an intermediate XML format, because I am interested in
    integration with my own custom XML document type in the future.

    <item>I use <name|Ruby> to generate the actual text, because the
    indentation handling is difficult to do from <name|XSLT>.
  </enumerate-numeric>

  <section|Implementation>

  <subsection|Step 1: Preprocessing>

  This is just a workaround for a couple of minor bugs in <TeXmacs>:

  <\enumerate-numeric>
    <item>A verbatim <verbatim|"> is output as <verbatim|\\"> in the
    <verbatim|.tmml> file. (This even happens when using the verbatim output
    method.) Ditto for <verbatim|{>,<verbatim|}>,<verbatim|\\>.

    <item>The XML escape sequences for <with|mode|math|\<less\>> and
    <with|mode|math|\<gtr\>> are <verbatim|&lt;> and <verbatim|&gt;>.

    <item>General Unicode characters should not be output using their entity
    names (e.g. <verbatim|&leftarrow;>) but using their numeric character
    code. Otherwise, as the entities are not defined in the output XML
    document, the document is not valid XML.
  </enumerate-numeric>

  <\file|preprocess.rb>
    #!/bin/ruby

    infile = File::new(ARGV[0])

    outfile = File.new(ARGV[1],"w")

    infile.each_line do \|line\|

    \ \ \ \ \ \ \ \ line.gsub! /\\\\"/ \ \ \ , '"'

    \ \ \ \ \ \ \ \ line.gsub! /\\\\\\\\/ \ \ \ , '\\\\'

    \ \ \ \ \ \ \ \ line.gsub! /&less;/ , '&lt;'

    \ \ \ \ \ \ \ \ line.gsub! /&gtr;/ \ , '&gt;'

    \ \ \ \ \ \ \ \ line.gsub! /&leftarrow;/ \ , '' <cmt|What is the
    character code of ``<with|mode|math|\<leftarrow\>>''?>

    \ \ \ \ \ \ \ \ outfile.write(line)

    end
  </file>

  <subsection|Step 2: Generating MyCode XML>

  <subsubsection|What is MyCode?>

  MyCode is a simple XML document type, which I use because\ 

  <\itemize>
    <item>it cleanly represents the kind of LP constructs I want to allow

    <item>it will integrate well with some other document types I use.
  </itemize>

  An example MyCode document, which says everything there is to know about
  MyCode, is

  <\file|mycode-example.xml>
    \<less\>?xml version="1.0" encoding="UTF-8"?\<gtr\>

    \;

    \<less\>document\<gtr\>

    \;

    \<less\>file name="f1.txt"\<gtr\>

    \ \ \<less\>block\<gtr\>

    \<less\>line\<gtr\>Line 1 is not indented.\<less\>/line\<gtr\>

    \ \ \ \ \<less\>line\<gtr\>Line 2 is indented in
    block.\<less\>/line\<gtr\>

    \<less\>line\<gtr\> \ \ \ Line 3 is indented in line.\<less\>/line\<gtr\>

    \ \ \ \ \<less\>line\<gtr\> \ \ \ Line 4 is indented in
    both.\<less\>/line\<gtr\>

    \ \ \<less\>/block\<gtr\>

    \<less\>/file\<gtr\>

    \;

    \<less\>block name="inner block"\<gtr\>

    \ \ \<less\>line\<gtr\>Inner Block Line One\<less\>/line\<gtr\>

    \ \ \<less\>line\<gtr\> \ Inner Block Line Two (indent:
    2)\<less\>/line\<gtr\>

    \ \ \<less\>line\<gtr\>Inner Block Line Three\<less\>/line\<gtr\>

    \<less\>/block\<gtr\>

    \;

    \<less\>block name="outer block"\<gtr\>

    \ \ \<less\>line\<gtr\>Outer Block Line One\<less\>/line\<gtr\>

    \ \ \<less\>line\<gtr\>And now to identical copies of the inner
    block\<less\>/line\<gtr\>

    \ \ \<less\>line\<gtr\>\<less\>inc name="inner
    block"/\<gtr\>\<less\>/line\<gtr\>

    \ \ \<less\>inc name="inner block"/\<gtr\>

    \<less\>/block\<gtr\>

    \;

    \<less\>file name="f2.txt"\<gtr\>

    \ \ \<less\>line\<gtr\>First comes the inner Block alone,
    \<less\>/line\<gtr\>

    \ \ \<less\>line\<gtr\>indented:\<less\>/line\<gtr\>

    \ \ \<less\>line\<gtr\> \ \ \ \<less\>inc name="inner
    block"/\<gtr\>\<less\>/line\<gtr\>

    \ \ \<less\>line\<gtr\>Then comes a line with four
    \ \ \ spaces.\<less\>/line\<gtr\>

    \ \ \<less\>line\<gtr\>And Finally, there comes the Outer
    Block\<less\>/line\<gtr\>

    \ \ \<less\>line\<gtr\> \ \ \ BEFORE \<less\>inc name="outer
    block"/\<gtr\> AFTER\<less\>/line\<gtr\>

    \<less\>/file\<gtr\>

    \;

    \<less\>/document\<gtr\>
  </file>

  <subsubsection|The Conversion Process>

  The conversion from <verbatim|tmml> to MyCode is done via an XSLT
  stylesheet called <verbatim|tmml2mycode.xsl>.

  <\file|tmml2mycode.xsl>
    \<less\>?xml version="1.0"?\<gtr\>

    \;

    \<less\>xsl:stylesheet version="1.0"

    \ \ \ \ xmlns:xsl="http://www.w3.org/1999/XSL/Transform"

    \ \ \ \ xmlns:exsl="http://exslt.org/common"\<gtr\>

    \;

    \<less\>xsl:output indent="no"/\<gtr\>

    \;

    <cmt|This kicks off the conversion process.>

    \<less\>xsl:template match="/"\<gtr\>

    \ \ \ \ \<less\>document\<gtr\>

    \ \ \ \ \ \ \ \ \<less\>xsl:apply-templates select="//file \|
    //block"/\<gtr\> \ \ \ \ \ \ \ \ 

    \ \ \ \ \<less\>/document\<gtr\>

    \<less\>/xsl:template\<gtr\>

    \;

    <inc|file and block templates>

    <inc|other templates>

    \;

    \<less\>/xsl:stylesheet\<gtr\>
  </file>

  \;

  The file and block templates function almost identically. This is where the
  real work is done: the positional arguments of <TeXmacs>' ``tags'' are
  mapped to XML's named arguments.

  <\block|file and block templates>
    \<less\>xsl:template match="file"\<gtr\>

    \ \ \ \ \ \ \ \ \<less\>file\<gtr\>

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \<less\>xsl:call-template
    name="file-or-block"/\<gtr\>

    \ \ \ \ \ \ \ \ \<less\>/file\<gtr\>

    \<less\>/xsl:template\<gtr\>

    \;

    \<less\>xsl:template match="block"\<gtr\>

    \ \ \ \ \ \ \ \ \<less\>block\<gtr\>

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \<less\>xsl:call-template
    name="file-or-block"/\<gtr\>

    \ \ \ \ \ \ \ \ \<less\>/block\<gtr\>

    \<less\>/xsl:template\<gtr\>

    \;

    \<less\>xsl:template name="file-or-block"\<gtr\>

    \ \ \ \ \<less\>xsl:attribute name="name"\<gtr\>

    \ \ \ \ \ \ \ \ \<less\>xsl:value-of select="./tm-arg[position()=1]/text()"/\<gtr\>

    \ \ \ \ \<less\>/xsl:attribute\<gtr\>

    \ \ \ \ \<less\>xsl:for-each select="./tm-arg[position()=2]/tm-par"\<gtr\>

    \ \ \ \ \ \ \ \ \<less\>line\<gtr\>

    \ \ \ \ \ \ \ \ \ \ \ \ \<less\>xsl:apply-templates\<gtr\>

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \<less\>xsl:with-param
    name="strip"\<gtr\>

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \<less\>xsl:choose\<gtr\>

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \<less\>xsl:when
    test="@xml:space='preserve'"\<gtr\>false\<less\>/xsl:when\<gtr\>

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \<less\>xsl:otherwise\<gtr\>true\<less\>/xsl:otherwise\<gtr\>

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \<less\>/xsl:choose\<gtr\>

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \<less\>/xsl:with-param\<gtr\>

    \ \ \ \ \ \ \ \ \ \ \ \ \<less\>/xsl:apply-templates\<gtr\>

    \ \ \ \ \ \ \ \ \<less\>/line\<gtr\>

    \ \ \ \ \<less\>/xsl:for-each\<gtr\>

    \<less\>/xsl:template\<gtr\>
  </block>

  \;

  The other templates do not do much. Most notable is the <verbatim|text()>
  template: <TeXmacs> does not add the <verbatim|xml:space="preserve">
  attribute to all tags contained in a verbatim environment. I.e.

  <\block|example for the XML export's whitespace handling>
    no indent

    \ single space indent
  </block>

  \;

  is rendered in <verbatim|tmml> as

  <\block-anon>
    ...

    \<less\>tm-arg\<gtr\>

    \ \ \ \<less\>tm-par\<gtr\>

    \ \ \ \ \ \ no indent

    \ \ \ \<less\>/tm-par\<gtr\>

    \ \ \ \<less\>tm-par xml:space="preserve"\<gtr\> single space
    indent\<less\>/tm-par\<gtr\>

    \<less\>/tm-arg\<gtr\>

    ...
  </block-anon>

  \;

  forcing me to <verbatim|normalize-space()>, which I would like to avoid.
  Anyway, this works for most purposes:

  <\block|other templates>
    \<less\>xsl:template match="text()"\<gtr\>

    \ \ \ \ \<less\>xsl:param name="strip"/\<gtr\>

    \ \ \ \ \<less\>xsl:choose\<gtr\>

    \ \ \ \ \ \ \ \ \<less\>xsl:when test="$strip='true'"\<gtr\>

    \ \ \ \ \ \ \ \ \ \ \ \ \<less\>xsl:value-of
    select="normalize-space()"/\<gtr\>

    \ \ \ \ \ \ \ \ \<less\>/xsl:when\<gtr\>

    \ \ \ \ \ \ \ \ \<less\>xsl:otherwise\<gtr\>

    \ \ \ \ \ \ \ \ \ \ \ \ \<less\>xsl:value-of select="."/\<gtr\>

    \ \ \ \ \ \ \ \ \<less\>/xsl:otherwise\<gtr\>

    \ \ \ \ \<less\>/xsl:choose\<gtr\>

    \<less\>/xsl:template\<gtr\>

    \;

    \<less\>xsl:template match="inc"\<gtr\>

    \ \ \ \ \<less\>inc\<gtr\>

    \ \ \ \ \ \ \ \ \<less\>xsl:attribute name="name"\<gtr\>

    \ \ \ \ \ \ \ \ \ \ \ \ \<less\>xsl:value-of select="./text()"/\<gtr\>

    \ \ \ \ \ \ \ \ \<less\>/xsl:attribute\<gtr\>

    \ \ \ \ \<less\>/inc\<gtr\>

    \<less\>/xsl:template\<gtr\>

    \;

    \<less\>xsl:template match="cmt"/\<gtr\>
  </block>

  <subsection|Step 3: Generating Code>

  Code is generated via a <name|Ruby> script. I decided against a XSLT
  stylesheet, because the indentation/whitespace handling required for
  <verbatim|\<less\>inc\<gtr\>>s is difficult to do in XSLT.

  Basically, every <verbatim|\<less\>block\<gtr\>>,
  <verbatim|\<less\>file\<gtr\>> or <verbatim|\<less\>line\<gtr\>> element is
  mapped recursively to a list of strings. This process is done once for
  every <verbatim|\<less\>file\<gtr\>> element, and the result is written to
  the according file.

  <\file|myx2code.rb>
    require "rexml/document"

    \;

    <inc|helper functions>

    \;

    def convert_file(filename,outdir="")

    \ \ \ \ \ \ \ \ file = File::open(filename)

    \ \ \ \ \ \ \ \ $doc = REXML::Document.new file

    \ \ \ \ \ \ \ \ file.close

    \;

    \ \ \ \ \ \ \ \ <cmt|get all file elements in document>

    \ \ \ \ \ \ \ \ file_elements = $doc.elements.to_a("//file")

    \ \ \ \ \ \ \ \ file_elements.each do \|fe\|

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ file =
    File::open(outdir+fe.attributes["name"],"w")

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ strs = block_as_strs(fe)

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ strs.each { \|s\| file.write(s+"\\n") }

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ file.close

    \ \ \ \ \ \ \ \ end

    end

    \;

    convert_file(ARGV[0])
  </file>

  \;

  As helper functions I have:

  <\description-compact>
    <item*|<verbatim|block_as_strs>>Maps a <verbatim|\<less\>block\<gtr\>> or
    <verbatim|\<less\>file\<gtr\>> element to a list of strings.

    <item*|<verbatim|line_as_strs>>Ditto for <verbatim|\<less\>line\<gtr\>>
    elements.

    <item*|<verbatim|dispatch>>Given a DOM node, <verbatim|dispatch> checks
    its type and calls the appropriate function, returning a list of strings.
    It handles <verbatim|text()> nodes, and <verbatim|line>, <verbatim|block>
    and <verbatim|inc> elements. All other elements are mapped to the empty
    list.
  </description-compact>

  <\block|helper functions>
    <inc|block handling>

    <inc|line handling>

    <inc|dispatch>
  </block>

  \;

  First we turn to <verbatim|dispatch>. Most importantly, upon encountering
  an <verbatim|inc> element, <verbatim|dispatch> looks up the corresponding
  block via <name|XPath> and recursively call <verbatim|dispatch> on that
  one.

  <\block|dispatch>
    def dispatch(node)

    \ \ \ \ \ \ \ \ if node.is_a? REXML::Text

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ arr = Array.new()

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ return arr \<less\>\<less\>
    REXML::Text::unnormalize(node.to_s)

    \ \ \ \ \ \ \ \ elsif node.is_a? REXML::Element

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ if node.name == "line"

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ return line_as_strs(node)

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ elsif node.name == "block"

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ return
    block_as_strs(node)

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ elsif node.name == "inc"

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ target =
    node.attributes["name"]

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ query =
    "//block[@name='#{target}']"

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ node =
    REXML::XPath.first($doc,query)

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ if node == nil

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ puts
    "Can't find include: "+target

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ return []

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ end

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ return dispatch(node)

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ else

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ return []

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ end

    \ \ \ \ \ \ \ \ end

    end
  </block>

  \;

  The definition of <verbatim|block_as_strs> is straightforward.

  <\block|block handling>
    def block_as_strs(element)

    \ \ \ \ \ \ \ \ result = []

    \ \ \ \ \ \ \ \ element.elements.each { \|e\| result.concat(dispatch(e))
    }

    \ \ \ \ \ \ \ \ return result

    end
  </block>

  \;

  Line handling in <verbatim|line_as_strs> is tricky. The reason for this is,
  that a single line may contain or <verbatim|inc>lude a <verbatim|block>
  element, which consists of several lines. So for each child node of the
  line, we recursively obtain a list of strings and all of these lists may
  differ in length. <verbatim|line_as_strs> pads the shorter lists until all
  lists have the same length.

  <\block|line handling>
    def line_as_strs(element)

    \ \ \ \ \ \ \ \ values = []

    \ \ \ \ \ \ \ \ REXML::XPath.each(element,'./(text()\|*)') do \|n\|

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ v = dispatch(n)

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ values \<less\>\<less\> v\ 

    \ \ \ \ \ \ \ \ end

    \ \ \ \ \ \ \ \ return [""] if values == []

    \ \ \ \ \ \ \ \ <cmt|How long are my arrays? How many lines do I need?>

    \ \ \ \ \ \ \ \ num_of_lines = values.map{\|v\|v.length}.max

    \ \ \ \ \ \ \ \ <cmt|This makes the arrays in <verbatim|values> all the
    same lenght,>

    \ \ \ \ \ \ \ \ <cmt|by adding copies of the last entry, if there is just
    one entry,>

    \ \ \ \ \ \ \ \ <cmt|and the empty string otherwise.>

    \ \ \ \ \ \ \ \ values.map! do \|v\|

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ filler = ""

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ filler = v[0] if v.length == 1

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ v.fill(filler,((v.length)..(num_of_lines-1)))

    \ \ \ \ \ \ \ \ end

    \ \ \ \ \ \ \ \ <cmt|This concatenates the rows (thinking of the entries
    of <verbatim|values> as columns).>

    \ \ \ \ \ \ \ \ result = []

    \ \ \ \ \ \ \ \ (0..num_of_lines-1).each do \|i\|

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ result \<less\>\<less\> ""

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ values.each do \|v\|

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ result[-1]
    \<less\>\<less\> v[i]

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ end

    \ \ \ \ \ \ \ \ end

    \ \ \ \ \ \ \ \ return result

    end
  </block>

  <section|To Do>

  <\description>
    <item*|Folding of Blocks and Outline Support>As all of this is inspired
    by <name|Leo>, outlining naturally is a feature I want. Folding, as a
    first step, may be doable without too much effort, based on the switches
    functionality already in <TeXmacs>.

    <item*|Autoindent and (Soft) Tabs>Struggling with code indentation is the
    most tedious aspect of editing code in <TeXmacs>. Autoindenting is beyond
    my ability, but defining a keystroke to insert <with|mode|math|n> spaces
    should be no problem.

    <item*|Reading Changes to the Generated Files Back into <TeXmacs>>When
    debugging a program, it is much more natural to edit the generated files
    directly. Another great feature of <name|Leo> is, that it automatically
    applys changes made to the generated files to the source file. This is
    doable by annotating the generated files intelligently. I might try to
    implement this, as I am thinking about a MyX to <verbatim|tmml>
    stylesheet as well.

    <item*|Keyboard and Menu Shortcuts>Shortcuts for inserting the
    environments and initiating the conversion process would be useful. They
    should be easy to do, though I could use some pointers here.

    <item*|Syntax-Highlighting>Very difficult to do, I guess. As this would
    require a lot of <TeXmacs> scripting, I won't attempt this.

    <item*|Prettier Environments>There is always that.
  </description>
</body>

<\references>
  <\collection>
    <associate|auto-1|<tuple|1|?>>
    <associate|auto-10|<tuple|2.3|?>>
    <associate|auto-11|<tuple|3|?>>
    <associate|auto-2|<tuple|1|?>>
    <associate|auto-3|<tuple|2|?>>
    <associate|auto-4|<tuple|3|?>>
    <associate|auto-5|<tuple|2|?>>
    <associate|auto-6|<tuple|2.1|?>>
    <associate|auto-7|<tuple|2.2|?>>
    <associate|auto-8|<tuple|2.2.1|?>>
    <associate|auto-9|<tuple|2.2.2|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|Overview>
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <pageref|auto-1><vspace|0.5fn>

      <with|par-left|<quote|6fn>|<with|font-size|<quote|0.84>|What LP
      Functionality is Provided? <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <pageref|auto-2><vspace|0.15fn>>>

      <with|par-left|<quote|6fn>|<with|font-size|<quote|0.84>|How Does the
      Code Generation Work? <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <pageref|auto-3><vspace|0.15fn>>>

      <with|par-left|<quote|6fn>|<with|font-size|<quote|0.84>|Why so Many
      Steps? <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <pageref|auto-4><vspace|0.15fn>>>

      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|Implementation>
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <pageref|auto-5><vspace|0.5fn>

      <with|par-left|<quote|1.5fn>|Step 1: Preprocessing
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <pageref|auto-6>>

      <with|par-left|<quote|1.5fn>|Step 2: Generating MyCode XML
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <pageref|auto-7>>

      <with|par-left|<quote|3fn>|What is MyCode?
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <pageref|auto-8>>

      <with|par-left|<quote|3fn>|The Conversion Process
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <pageref|auto-9>>

      <with|par-left|<quote|1.5fn>|Step 3: Generating Code
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <pageref|auto-10>>

      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|To
      Do> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <pageref|auto-11><vspace|0.5fn>
    </associate>
  </collection>
</auxiliary>